local Mounted = Mounted
local L = LibStub("AceLocale-3.0"):GetLocale("Mounted", false)

Mounted.DescOption = {
	type="group",
	width = 'fill',
	handler = Mounted,
	childGroups = 'tab',
	args={
		f = {
			name = L['Flight'],
			desc = L['This Macro will be used in any location you may fly!'],
			type="group",
			order = 1,
			args={
			},
		},
		g = {
			name = L['Ground'],
			desc = L['This Macro will be used in any location you may ride!'],
			type="group",
			order = 2,
			args={
			},
		},
		aq = {
			name = L["Ahn'Qiraj"],
			desc = L["This Macro will be used in Ahn'Qiraj"],
			type="group",
			order = 3,
			args={
			},
		},
		cm = {
			name = L["Combat"],
			desc = L["This Macro will be used in Combat and Indoors!"],
			type="group",
			order = 4,
			args={
			},
		},
	},
}

local db = {
	["char"] = L['char'],
	["class"] = L['class'],
	["race"] = L['race'],
	['global'] = L['global'],
}

local MacroOptions = {
	select={
		name=L['Select'],
		type="select",
		order = 10,
		values = function(info)
			local v = {}
			for k in pairs(Mounted.db.global.macros[info[#info-1]]) do
				if k == 'Default' or k == "Default Dismount" then
					v[k] = L[k]
				else
					v[k] = k
				end
			end
			v['new'] = L["New"]
			return v
		end,
		get = 'OGet',
		set = 'OSet',
	},
	name={
		name=L['Name'],
		type="input",
		order = 20,
		get = 'OGet',
		set = 'OSet',
		validate = function(info,text)
			if Mounted.db.global.macros[info[#info-1]][text] then
				Mounted:Print(string.format(L["NameError"], text))
				return string.format(L["NameError"], text)
			else
				return true
			end
		end,
		disabled = 'ODisabled',
	},
	macro={
		name=L['Macro'],
		desc=L["FAQdesc"],
		type="input",
		multiline  = true,
		width = 'full',
		order = 30,
		get = 'OGet',
		set = 'OSet',
		disabled = 'ODisabled',
	},
	preview={
		type = 'toggle',
		order = 40,
		name = L['Preview'],
		desc = L['Toggle Preview'],
		get = 'OGet',
		set = 'OSet',
	},
	linebreak1 = {
		name = '',
		order = 45,
		type="description",
	},
	set = {
		name=L['Set Macro'],
		desc=L['Will set the macro to the selected one.\nWill also save the macro.'],
		type="execute",
		order = 50,
		func = 'Ofunc',
		disabled = 'ODisabled',
	},
	reset = {
		name=L['Reset Macro'],
		desc=L['Will discard all changes and reset the macro to the previous one.'],
		type="execute",
		order = 60,
		func = 'Ofunc',
		disabled = 'ODisabled',
	},
	delete = {
		name=L['Delete Macro'],
		desc=L['Will delete the selected macro and revert to the default macro.'],
		type="execute",
		order = 70,
		func = 'Ofunc',
		disabled = 'ODisabled',
	},
	linebreak2 = {
		name = '',
		order = 80,
		type="description",
	},
	db={
		name=L['db'],
		type="select",
		order = 90,
		values = db,
		get = 'OGet',
		set = 'OSet',
	},
}

Mounted.f = {}
Mounted.g = {}
Mounted.aq = {}
Mounted.cm = {}

function Mounted:OGet(info)
	if info[#info] == 'select' or info[#info] == 'name' then
		if not self[info[#info-1]].name then
			self[info[#info-1]].name = self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]
		end
		return self[info[#info-1]].name
	elseif info[#info] == 'db' then
		return self.db.char[info[#info-1]..'loc']
	elseif info[#info] == 'macro' then
		if not self[info[#info-1]].macro then
			self[info[#info-1]].macro = self.db.global.macros[info[#info-1]][self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]]
		end
		if self[info[#info-1]].preview then
			return self:ParseMacro(self[info[#info-1]].macro, true)
		end
		return self[info[#info-1]].macro
	elseif info[#info] == 'preview' then
		return self[info[#info-1]].preview
	end
end

function Mounted:OSet(info, key)
	if info[#info] == 'select' then
		if key == 'new' then
			self[info[#info-1]].name = ''
			self[info[#info-1]].macro = ''
			self[info[#info-1]].preview = nil
		else
			self[info[#info-1]].name = key
			self[info[#info-1]].macro = self.db.global.macros[info[#info-1]][key]
		end
	elseif info[#info] == 'name' then
		self[info[#info-1]].name = key
	elseif info[#info] == 'db' then
		if not self.db[key][info[#info-1]] then
			self.db[key][info[#info-1]] = self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]]
		end
		self.db.char[info[#info-1]..'loc'] = key
	elseif info[#info] == 'macro' then
		self[info[#info-1]].macro = key
	elseif info[#info] == 'preview' then
		self[info[#info-1]].preview = key
	end
end

function Mounted:Ofunc(info)
	if info[#info] == 'set' then
		self.db.global.macros[info[#info-1]][self[info[#info-1]].name] = self[info[#info-1]].macro
		self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] = self[info[#info-1]].name
	elseif info[#info] == 'reset' then
		self[info[#info-1]].name = nil
		self[info[#info-1]].macro = nil
	elseif info[#info] == 'delete' then
		if self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] == self[info[#info-1]].name then
			self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] = 'Default'
		end
		self.db.global.macros[info[#info-1]][self[info[#info-1]].name] = nil
		self[info[#info-1]].name = nil
		self[info[#info-1]].macro = nil
	end
end

function Mounted:ODisabled(info)
	if info[#info] == 'name' or info[#info] == 'delete' then
		if self.locked[info[#info-1]][self[info[#info-1]].name] then
			return true
		end
	elseif info[#info] == 'macro' then
		if self.locked[info[#info-1]][self[info[#info-1]].name] or self[info[#info-1]].preview then
			return true
		end
	elseif info[#info] == 'set' then
		if self[info[#info-1]].name == '' or self[info[#info-1]].macro == '' or self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] == self[info[#info-1]].name then
			return true
		end
	elseif info[#info] == 'reset' then
		if self.db[self.db.char[info[#info-1]..'loc']][info[#info-1]] == self[info[#info-1]].name then
			return true
		end
	end
	return nil
end

Mounted.DescOption.args.g.args = MacroOptions
Mounted.DescOption.args.f.args = MacroOptions
Mounted.DescOption.args.aq.args = MacroOptions
Mounted.DescOption.args.cm.args = MacroOptions

function Mounted:UpdateOptions()
	self:RangeCategory('f', '310')
	self:RangeCategory('f', '280')
	self:RangeCategory('f', '60')
	self:RangeCategory('f', 's')
	self:RangeCategory('g', '100')
	self:RangeCategory('g', '60')
	self:RangeCategory('g', '0')
	self:RangeCategory('g', 'aq')
end

function Mounted:RangeCategory(mtype, mspeed)
	local mounts = {}
	local RangeTable = {	
		type="group",
		width = 'fill',
		handler = Mounted,
		args={
		},
	}
	for k in pairs(self.mounts[mtype][mspeed]) do
		mounts[#mounts + 1] = k
	end
	table.sort(mounts)
	for i, mountname in ipairs(mounts) do
		RangeTable.args[string.format('%d_%s_%s', i, mtype, mspeed)] = self:CreateRange(mountname, i)
	end
	local optionname = string.format(L["%s: %s"], L[mtype], L[mspeed])
	if next(RangeTable.args) then
		self:RegisterModuleOptions(optionname, RangeTable, optionname)
	end
end

function Mounted:GetRange(info, info2)
	local mtype, mspeed = string.match(info[#info], '%d+_(.)_(.+)')
	return self.mounts[mtype][mspeed][info.option.name]
end

function Mounted:SetRange(info, num)
	local mtype, mspeed = string.match(info[#info], '%d+_(.)_(.+)')
	self.mounts[mtype][mspeed][info.option.name] = num
end

function Mounted:CreateRange(mountname,ordernum)
	local Range = {
			type = 'range',
			name = mountname,
			order = ordernum,
			width = 'full',
			min = 0,
			max = 10,
			step = 1,
			get = 'GetRange',
			set = 'SetRange',
		}
	return Range
end

Mounted.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('Mounted', Mounted.DescOption)
Mounted.optframe.Mounted = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('Mounted', 'Mounted')
-- function addapted from the RegisterModuleOptions function in Mapster
function Mounted:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "Mounted"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "Mounted")
end
